local folder, core = ...
_G["_CLV"] = core

local curseVersion = GetAddOnMetadata(folder, "X-Curse-Packaged-Version")
local curseRevision = GetAddOnMetadata(folder, "X-Curse-Packaged-Revision") -- 19
if curseVersion and curseRevision and curseVersion ~= "r"..curseRevision then
	curseVersion = curseVersion.." (r"..curseRevision..")" --alpha versions will show "r99", tagged versions will show "1.2.3beta (r99)"
end

core.title		= GetAddOnMetadata(folder, "Title")
core.version	= curseVersion or "[dev]"
core.titleFull	= core.title.." "..(curseVersion and "v"..tostring(core.version) or tostring(core.version))
core.addonDir = "Interface\\AddOns\\"..folder.."\\"

LibStub("AceAddon-3.0"):NewAddon(core, folder, "AceConsole-3.0", "AceHook-3.0")

local L = LibStub("AceLocale-3.0"):GetLocale(folder, true)

core.defaultSettings = {
	profile = {},
	factionrealm = {}, --Used for cached prices.
}


local tostring = tostring
local chatFrame = _G["ChatFrame1"]

local DEBUG = false
--[===[@debug@
DEBUG = true
--@end-debug@]===]
local strWhiteBar		= "|cffffff00 || |r" -- a white bar to seperate the debug info.
local colouredName		= "|cff7f7f7f{|r|cffff0000CLV|r|cff7f7f7f}|r "
local function echo(...)
	local tbl  = {...}
	local msg = tostring(tbl[1])
	for i=2,#tbl do 
		msg = msg..strWhiteBar..tostring(tbl[i])
	end

	local cf = chatFrame
	if cf then
		cf:AddMessage(colouredName..msg,.7,.7,.7)
	end
end
core.echo = echo

local strDebugFrom		= "|cffffff00[%s]|r" --Yellow function name. help pinpoint where the debug msg is from.
-----------------------------
local function Debug(from, ...)	--
-- simple print function.	--
------------------------------
--~ 	if DEBUG == false then
--~ 		return 
--~ 	end
	if core.db.profile.debugMessages == false then
		return
	end
	local tbl  = {...}
	local msg = tostring(tbl[1])
	for i=2,#tbl do 
		msg = msg..strWhiteBar..tostring(tbl[i])
	end
	echo(strDebugFrom:format(from).." "..msg)
end
core.Debug = Debug

do
	local LibStub = LibStub
	function core:OnInitialize()
		self.db = LibStub("AceDB-3.0"):New("CLV_DB", core.defaultSettings, true) --'Default'
	
		self.db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
		self.db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
		self.db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")
		self.db.RegisterCallback(self, "OnProfileDeleted", "OnProfileChanged")
	end
end

----------------------------------------------------------------------
function core:OnProfileChanged(...)									--
-- User has reset proflie, so we reset our spell exists options.	--
----------------------------------------------------------------------
--~ 	-- Shut down anything left from previous settings
	self:Disable()
--~ 	-- Enable again with the new settings
	self:Enable()
end

do
	local pairs = pairs
	function core:OnEnable()
		core:RawHook("ChatFrame_OnEvent", true)
	end
end

local LOOT_ITEM = LOOT_ITEM:gsub("%%s", "(.+)")												--"%s receives loot: %s."
local LOOT_ITEM_CREATED_SELF = LOOT_ITEM_CREATED_SELF:gsub("%%s", "(.+)") 					--"You create: %s."
local LOOT_ITEM_CREATED_SELF_MULTIPLE = LOOT_ITEM_CREATED_SELF_MULTIPLE:gsub("%%s", "(.+)")
local LOOT_ITEM_CREATED_SELF_MULTIPLE = LOOT_ITEM_CREATED_SELF_MULTIPLE:gsub("%%d", "(.+)")	--"You create: %sx%d."
local LOOT_ITEM_MULTIPLE = LOOT_ITEM_MULTIPLE:gsub("%%s", "(.+)")
local LOOT_ITEM_MULTIPLE = LOOT_ITEM_MULTIPLE:gsub("%%d", "(.+)")							--"%s receives loot: %sx%d."
local LOOT_ITEM_PUSHED_SELF = LOOT_ITEM_PUSHED_SELF:gsub("%%s", "(.+)")						--"You receive item: %s."
local LOOT_ITEM_PUSHED_SELF_MULTIPLE = LOOT_ITEM_PUSHED_SELF_MULTIPLE:gsub("%%s", "(.+)")
local LOOT_ITEM_PUSHED_SELF_MULTIPLE = LOOT_ITEM_PUSHED_SELF_MULTIPLE:gsub("%%d", "(.+)")	--"You receive item: %sx%d."
local LOOT_ITEM_SELF = LOOT_ITEM_SELF:gsub("%%s", "(.+)")									--"You receive loot: %s."
local LOOT_ITEM_SELF_MULTIPLE = LOOT_ITEM_SELF_MULTIPLE:gsub("%%s", "(.+)")
local LOOT_ITEM_SELF_MULTIPLE = LOOT_ITEM_SELF_MULTIPLE:gsub("%%d", "(.+)")					--"You receive loot: %sx%d."


do
	local strChatIcon	= "|T%s:0:0:0:0|t"; --sending icon to local chat, CANT be sent threw chat channels to others.
	function core:ChatIcon(texture)
		return strChatIcon:format(texture)
	end
end

do
	local strsplit = strsplit
	local tonumber = tonumber
	
	local _, itemString, itemId
	local function ItemLinkToId(itemLink) -- Itemlink, returns the itemID
		if itemLink then
			_, _, itemString = itemLink:find("^|c%x+|H(.+)|h%[.+%]")
			if itemString then
				_, itemId = strsplit(":", itemString)
				return tonumber(itemId)
			end
		end
		return 0
	end

	core.iID = setmetatable({}, {
		__index = function(t,i)
		-- Get the value and stash it for later
		local v = ItemLinkToId(i)
		t[i] = v
		return v
		end
	})
end

do
	local GetItemIcon = GetItemIcon
	
	local find, left, right
	function core:AddItemIcon(msg, link)
		find = msg:find(":")
		left = msg:sub(1, find+1)
		right = msg:sub(find+2, msg:len())
		
		return left..self:ChatIcon( GetItemIcon(link) )..right
	end
end

do
	local tonumber = tonumber

	function core:ChatFrame_OnEvent(frame, event, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15)
		if event == "CHAT_MSG_LOOT" then
			for player, link, stack in arg1:gmatch(LOOT_ITEM_MULTIPLE) do
				if self.db.profile.addItemIcon == true then
					arg1 = self:AddItemIcon(arg1, link)
				end
				arg1 = arg1.." "..self:GetValueString(link, stack);
				self.hooks.ChatFrame_OnEvent(frame, event, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15)
				return
			end
		
			for player, link in arg1:gmatch(LOOT_ITEM) do
				if self.db.profile.addItemIcon == true then
					arg1 = self:AddItemIcon(arg1, link)
				end
				arg1 = arg1.." "..self:GetValueString(link);
				self.hooks.ChatFrame_OnEvent(frame, event, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15)
				return
			end

			for link, stack in arg1:gmatch(LOOT_ITEM_CREATED_SELF_MULTIPLE) do
				if self.db.profile.addItemIcon == true then
					arg1 = self:AddItemIcon(arg1, link)
				end
				arg1 = arg1.." "..self:GetValueString(link, stack);
				self.hooks.ChatFrame_OnEvent(frame, event, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15)
				return
			end
			
			for link in arg1:gmatch(LOOT_ITEM_CREATED_SELF) do
				if self.db.profile.addItemIcon == true then
					arg1 = self:AddItemIcon(arg1, link)
				end
				arg1 = arg1.." "..self:GetValueString(link);
				self.hooks.ChatFrame_OnEvent(frame, event, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15)
				return
			end

			for link, stack in arg1:gmatch(LOOT_ITEM_PUSHED_SELF_MULTIPLE) do
				if self.db.profile.addItemIcon == true then
					arg1 = self:AddItemIcon(arg1, link)
				end
				arg1 = arg1.." "..self:GetValueString(link, stack);
				self.hooks.ChatFrame_OnEvent(frame, event, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15)
				return
			end
			
			for link in arg1:gmatch(LOOT_ITEM_PUSHED_SELF) do
				if self.db.profile.addItemIcon == true then
					arg1 = self:AddItemIcon(arg1, link)
				end
				arg1 = arg1.." "..self:GetValueString(link);
				self.hooks.ChatFrame_OnEvent(frame, event, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15)
				return
			end
			
			for link, stack in arg1:gmatch(LOOT_ITEM_SELF_MULTIPLE) do
				if self.db.profile.addItemIcon == true then
					arg1 = self:AddItemIcon(arg1, link)
				end
				arg1 = arg1.." "..self:GetValueString(link, stack);
				self.hooks.ChatFrame_OnEvent(frame, event, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15)
				return
			end
			
			for link in arg1:gmatch(LOOT_ITEM_SELF) do
				if self.db.profile.addItemIcon == true then
					arg1 = self:AddItemIcon(arg1, link)
				end
				arg1 = arg1.." "..self:GetValueString(link);
				self.hooks.ChatFrame_OnEvent(frame, event, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15)
				return
			end

		end

		self.hooks.ChatFrame_OnEvent(frame, event, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15)
	end
end